@@ -176,7 +176,7 @@ Results *FindNormalRoute(Nodes *nodes,Segments *segments,Ways *ways,Relations *r
 
           wayp=LookupWay(ways,segmentp->way,1);
 
-          if(!(wayp->props&Properties_CycleBothWays))
+          if(!(wayp->type&Highway_CycleBothWays))
              goto endloop;
          }
 
@@ -236,10 +236,13 @@ Results *FindNormalRoute(Nodes *nodes,Segments *segments,Ways *ways,Relations *r
           goto endloop;
 
        for(i=1;i<Property_Count;i++)
-          if(wayp->props & PROPERTIES(i))
-             segment_pref*=profile->props_yes[i];
-          else
-             segment_pref*=profile->props_no[i];
+          if(ways->file.props & PROPERTIES(i))
+            {
+             if(wayp->props & PROPERTIES(i))
+                segment_pref*=profile->props_yes[i];
+             else
+                segment_pref*=profile->props_no[i];
+            }
 
        /* profile preferences must allow this highway */
        if(segment_pref==0)
@@ -411,7 +414,7 @@ Results *FindMiddleRoute(Nodes *nodes,Segments *segments,Ways *ways,Relations *r
  while(result3)
    {
     if((results->start_node!=result3->node || results->prev_segment!=result3->segment) &&
-       !IsFakeNode(result3->node) && IsSuperNode(LookupNode(nodes,result3->node,5)))
+       !IsFakeNode(result3->node) && IsSuperNode(LookupNode(nodes,result3->node,3)))
       {
        Result *result5=result1;
        index_t superseg=FindSuperSegment(nodes,segments,ways,relations,profile,result3->node,result3->segment);
@@ -420,6 +423,8 @@ Results *FindMiddleRoute(Nodes *nodes,Segments *segments,Ways *ways,Relations *r
          {
           result5=InsertResult(results,result3->node,result3->segment);
 
+          result5->score=result3->score;
+
           result5->prev=result1;
          }
 
@@ -505,7 +510,7 @@ Results *FindMiddleRoute(Nodes *nodes,Segments *segments,Ways *ways,Relations *r
 
           wayp=LookupWay(ways,segmentp->way,1);
 
-          if(!(wayp->props&Properties_CycleBothWays))
+          if(!(wayp->type&Highway_CycleBothWays))
              goto endloop;
          }
 
@@ -549,10 +554,13 @@ Results *FindMiddleRoute(Nodes *nodes,Segments *segments,Ways *ways,Relations *r
           goto endloop;
 
        for(i=1;i<Property_Count;i++)
-          if(wayp->props & PROPERTIES(i))
-             segment_pref*=profile->props_yes[i];
-          else
-             segment_pref*=profile->props_no[i];
+          if(ways->file.props & PROPERTIES(i))
+            {
+             if(wayp->props & PROPERTIES(i))
+                segment_pref*=profile->props_yes[i];
+             else
+                segment_pref*=profile->props_no[i];
+            }
 
        /* profile preferences must allow this highway */
        if(segment_pref==0)
@@ -707,15 +715,15 @@ static index_t FindSuperSegment(Nodes *nodes,Segments *segments,Ways *ways,Relat
  if(IsFakeSegment(finish_segment))
     finish_segment=IndexRealSegment(finish_segment);
 
- supernodep=LookupNode(nodes,finish_node,5); /* finish_node cannot be a fake node (must be a super-node) */
- supersegmentp=LookupSegment(segments,finish_segment,2); /* finish_segment cannot be a fake segment. */
+ supernodep=LookupNode(nodes,finish_node,3); /* finish_node cannot be a fake node (must be a super-node) */
+ supersegmentp=LookupSegment(segments,finish_segment,4); /* finish_segment cannot be a fake segment. */
 
  if(IsSuperSegment(supersegmentp))
     return(finish_segment);
 
  /* Loop across all segments */
 
- supersegmentp=FirstSegment(segments,supernodep,3); /* supernode cannot be a fake node (must be a super-node) */
+ supersegmentp=FirstSegment(segments,supernodep,4); /* supernode cannot be a fake node (must be a super-node) */
 
  while(supersegmentp)
    {
@@ -805,11 +813,11 @@ static Results *FindSuperRoute(Nodes *nodes,Segments *segments,Ways *ways,Relati
     node1=result1->node;
     seg1=result1->segment;
 
-    node1p=LookupNode(nodes,node1,1); /* node1 cannot be a fake node */
+    node1p=LookupNode(nodes,node1,3); /* node1 cannot be a fake node */
 
     /* Loop across all segments */
 
-    segmentp=FirstSegment(segments,node1p,1); /* node1 cannot be a fake node */
+    segmentp=FirstSegment(segments,node1p,3); /* node1 cannot be a fake node */
 
     while(segmentp)
       {
@@ -829,9 +837,9 @@ static Results *FindSuperRoute(Nodes *nodes,Segments *segments,Ways *ways,Relati
           if(profile->allow!=Transports_Bicycle)
              goto endloop;
 
-          wayp=LookupWay(ways,segmentp->way,1);
+          wayp=LookupWay(ways,segmentp->way,2);
 
-          if(!(wayp->props&Properties_CycleBothWays))
+          if(!(wayp->type&Highway_CycleBothWays))
              goto endloop;
          }
 
@@ -843,7 +851,7 @@ static Results *FindSuperRoute(Nodes *nodes,Segments *segments,Ways *ways,Relati
 
        node2=OtherNode(segmentp,node1);
 
-       node2p=LookupNode(nodes,node2,2); /* node2 cannot be a fake node */
+       node2p=LookupNode(nodes,node2,4); /* node2 cannot be a fake node */
 
        /* must not pass over super-node */
        if(node2!=finish_node && IsSuperNode(node2p))
@@ -1003,7 +1011,7 @@ Results *FindStartRoutes(Nodes *nodes,Segments *segments,Ways *ways,Relations *r
 
           wayp=LookupWay(ways,segmentp->way,1);
 
-          if(!(wayp->props&Properties_CycleBothWays))
+          if(!(wayp->type&Highway_CycleBothWays))
              goto endloop;
          }
 
@@ -1056,10 +1064,13 @@ Results *FindStartRoutes(Nodes *nodes,Segments *segments,Ways *ways,Relations *r
           goto endloop;
 
        for(i=1;i<Property_Count;i++)
-          if(wayp->props & PROPERTIES(i))
-             segment_pref*=profile->props_yes[i];
-          else
-             segment_pref*=profile->props_no[i];
+          if(ways->file.props & PROPERTIES(i))
+            {
+             if(wayp->props & PROPERTIES(i))
+                segment_pref*=profile->props_yes[i];
+             else
+                segment_pref*=profile->props_no[i];
+            }
 
        /* profile preferences must allow this highway */
        if(segment_pref==0)
@@ -1240,7 +1251,7 @@ Results *ExtendStartRoutes(Nodes *nodes,Segments *segments,Ways *ways,Relations
          }
 
     if(!IsFakeNode(result3->node))
-       if(IsSuperNode(LookupNode(nodes,result3->node,5)))
+       if(IsSuperNode(LookupNode(nodes,result3->node,3)))
           InsertInQueue(queue,result3,result3->score);
 
     result3=NextResult(results,result3);
@@ -1303,7 +1314,7 @@ Results *ExtendStartRoutes(Nodes *nodes,Segments *segments,Ways *ways,Relations
 
           wayp=LookupWay(ways,segmentp->way,1);
 
-          if(!(wayp->props&Properties_CycleBothWays))
+          if(!(wayp->type&Highway_CycleBothWays))
              goto endloop;
          }
 
@@ -1349,10 +1360,13 @@ Results *ExtendStartRoutes(Nodes *nodes,Segments *segments,Ways *ways,Relations
           goto endloop;
 
        for(i=1;i<Property_Count;i++)
-          if(wayp->props & PROPERTIES(i))
-             segment_pref*=profile->props_yes[i];
-          else
-             segment_pref*=profile->props_no[i];
+          if(ways->file.props & PROPERTIES(i))
+            {
+             if(wayp->props & PROPERTIES(i))
+                segment_pref*=profile->props_yes[i];
+             else
+                segment_pref*=profile->props_no[i];
+            }
 
        /* profile preferences must allow this highway */
        if(segment_pref==0)
@@ -1489,7 +1503,7 @@ Results *FindFinishRoutes(Nodes *nodes,Segments *segments,Ways *ways,Relations *
     node1=result1->node;
     seg1=result1->segment;
 
-    if(IsFakeSegment(seg1))
+    if(seg1!=NO_SEGMENT && IsFakeSegment(seg1))
        seg1r=IndexRealSegment(seg1);
     else
        seg1r=seg1;
@@ -1532,7 +1546,7 @@ Results *FindFinishRoutes(Nodes *nodes,Segments *segments,Ways *ways,Relations *
 
           wayp=LookupWay(ways,segmentp->way,1);
 
-          if(!(wayp->props&Properties_CycleBothWays))
+          if(!(wayp->type&Highway_CycleBothWays))
              goto endloop;
          }
 
@@ -1550,11 +1564,24 @@ Results *FindFinishRoutes(Nodes *nodes,Segments *segments,Ways *ways,Relations *
          }
 
        /* must not perform U-turn (unless profile allows) */
-       if(profile->turns && (seg1==seg2 || seg1==seg2r || seg1r==seg2 || (seg1r==seg2r && IsFakeUTurn(seg1,seg2))))
-          goto endloop;
+       if(profile->turns && seg1!=NO_SEGMENT)
+         {
+          if(IsFakeNode(node1) || !IsSuperNode(node1p))
+            {
+             if(seg1==seg2 || seg1==seg2r || seg1r==seg2 || (seg1r==seg2r && IsFakeUTurn(seg1,seg2)))
+                goto endloop;
+            }
+          else
+            {
+             index_t superseg=FindSuperSegment(nodes,segments,ways,relations,profile,node1,seg1);
+
+             if(seg2==superseg)
+                goto endloop;
+            }
+         }
 
        /* must obey turn relations */
-       if(turnrelation!=NO_RELATION)
+       if(turnrelation!=NO_RELATION && seg1!=NO_SEGMENT)
          {
           index_t turnrelation2=FindFirstTurnRelation2(relations,node1,seg2r); /* node2 -> node1 -> result1->next->node */
 
@@ -1585,10 +1612,13 @@ Results *FindFinishRoutes(Nodes *nodes,Segments *segments,Ways *ways,Relations *
           goto endloop;
 
        for(i=1;i<Property_Count;i++)
-          if(wayp->props & PROPERTIES(i))
-             segment_pref*=profile->props_yes[i];
-          else
-             segment_pref*=profile->props_no[i];
+          if(ways->file.props & PROPERTIES(i))
+            {
+             if(wayp->props & PROPERTIES(i))
+                segment_pref*=profile->props_yes[i];
+             else
+                segment_pref*=profile->props_no[i];
+            }
 
        /* profile preferences must allow this highway */
        if(segment_pref==0)
