@@ -279,6 +279,8 @@ function form_init()            // called from router.html
    }
 
  updateURLs();
+
+ updateSearchButtons();
 }
 
 
@@ -932,6 +934,8 @@ function markerToggleMap(marker) // called from router.html
    }
 
  markerAddRemoveMap(marker,!routino.point[marker].active);
+
+ updateSearchButtons();
 }
 
 
@@ -1065,6 +1069,8 @@ function markerRemove(marker)   // called from router.html
 
  if(vismarkers==1)
     markerAddAfter(1);
+
+ updateSearchButtons();
 }
 
 
@@ -1074,11 +1080,11 @@ function markerRemove(marker)   // called from router.html
 
 function markerAddBefore(marker)
 {
- clearSearchResult(marker);
-
  if(vismarkers==mapprops.maxmarkers || marker==1)
     return false;
 
+ clearSearchResult(marker);
+
  markerAddForm(++vismarkers);
 
  for(var m=vismarkers;m>marker;m--)
@@ -1094,11 +1100,11 @@ function markerAddBefore(marker)
 
 function markerAddAfter(marker) // called from router.html
 {
- clearSearchResult(marker);
-
  if(vismarkers==mapprops.maxmarkers)
     return false;
 
+ clearSearchResult(marker);
+
  markerAddForm(++vismarkers);
 
  for(var m=vismarkers;m>(marker+1);m--)
@@ -1144,6 +1150,31 @@ function markerLocate(marker)   // called from router.html
 }
 
 
+//
+// Update the search buttons enable/disable.
+//
+
+function updateSearchButtons()
+{
+ var markersactive=0;
+
+ for(var m=1;m<=vismarkers;m++)
+    if(routino.point[m].active)
+       markersactive++;
+
+ if(markersactive<2)
+   {
+    document.getElementById("shortest").disabled="disabled";
+    document.getElementById("quickest").disabled="disabled";
+   }
+ else
+   {
+    document.getElementById("shortest").disabled="";
+    document.getElementById("quickest").disabled="";
+   }
+}
+
+
 //
 // Update an icon to set colours and home or normal marker.
 //
@@ -1360,7 +1391,15 @@ function markersLoop()          // called from router.html
     return false;
 
  if(routino.point[vismarkers].lon==routino.point[1].lon && routino.point[vismarkers].lat==routino.point[1].lat)
-    return false;
+   {
+    if(routino.point[vismarkers].active)
+       return false;
+    else
+      {
+       markerToggleMap(vismarkers);
+       return true;
+      }
+   }
 
  if(routino.point[vismarkers].used)
     markerAddForm(++vismarkers);
@@ -1370,6 +1409,8 @@ function markersLoop()          // called from router.html
  markersmoved=true;
 
  updateURLs();
+
+ updateSearchButtons();
 }
 
 
@@ -1411,6 +1452,7 @@ function markerClearForm(marker)
 
  routino.point[marker].used=false;
  routino.point[marker].home=false;
+ routino.point[marker].active=false;
 }
 
 
